package config

import (
	"cicv-data-closedloop/aarch64/pjibot_patrol/common/config"
	"cicv-data-closedloop/common/config/c_log"
	"cicv-data-closedloop/common/util"
	"cicv-data-closedloop/pjibot_patrol_msgs"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/diagnostic_msgs"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/geometry_msgs"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/nav_msgs"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/sensor_msgs"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/std_msgs"
	"plugin"
	"slices"
)

func InitTriggerConfig() {
	config.OssMutex.Lock()
	defer config.OssMutex.Unlock()
	triggerLocalPathsMapTriggerId := make(map[string]string)
	c_log.GlobalLogger.Info("主节点加载触发器插件 - 开始。")
	// 1 获取数采任务的触发器列表
	cloudTriggers := &config.PlatformConfig.TaskTriggers
	// 2 获取本地触发器列表（触发器目录的一级子目录为【触发器ID_触发器Label】）
	localTriggerIds := util.GetFirstLevelSubdirectories(config.CloudConfig.TriggersDir)
	// 3 对比触发器列表，本地没有的则下载
	for _, trigger := range *cloudTriggers {
		id := util.ToString(trigger.TriggerId)
		hasIdDir := slices.Contains(localTriggerIds, id) // 改成了 slices 工具库
		triggerLocalDir := config.CloudConfig.TriggersDir + id + "/"
		hasLabelSo, soPaths := util.CheckSoFilesInDirectory(triggerLocalDir)
		var triggerLocalPath string
		if hasIdDir && hasLabelSo { // 已存在的触发器需要判断是否大小一致
			triggerLocalPath = soPaths[0]
			ossSize, _ := util.GetOSSFileSize(config.OssBucket, trigger.TriggerScriptPath)
			localSize, _ := util.GetFileSize(triggerLocalPath)
			if ossSize == localSize {
				c_log.GlobalLogger.Info("触发器插件 ", triggerLocalPath, " 存在且与云端触发器大小一致。")
				triggerLocalPathsMapTriggerId[triggerLocalPath] = id
				continue
			}
		}
		label := util.GetFileNameWithoutExtension(config.CloudConfig.TriggersDir + trigger.TriggerScriptPath)
		triggerLocalPath = config.CloudConfig.TriggersDir + id + "/" + label + ".so"
		c_log.GlobalLogger.Info("开始下载触发器插件从 ", trigger.TriggerScriptPath, " 到 ", triggerLocalPath)
		_ = util.CreateParentDir(triggerLocalPath)
		for {
			if err := config.OssBucket.GetObjectToFile(trigger.TriggerScriptPath, triggerLocalPath); err != nil {
				c_log.GlobalLogger.Error("下载触发器插件失败，再次尝试：", err)
				continue
			}
			break
		}
		triggerLocalPathsMapTriggerId[triggerLocalPath] = id
	}

	success := 0
	// 加载所有触发器的文件
	for triggerLocalPath, triggerId := range triggerLocalPathsMapTriggerId {
		// 载入插件到数组
		open, err := plugin.Open(triggerLocalPath)
		if err != nil {
			c_log.GlobalLogger.Error("加载本地插件", triggerLocalPath, "失败。", err)
			continue
		}
		topic0, err := open.Lookup("Topic")
		if err != nil {
			c_log.GlobalLogger.Error("加载本地插件", triggerLocalPath, "中的Topic方法失败。", err)
			continue
		}
		topic1, ok := topic0.(func() string)
		if ok != true {
			c_log.GlobalLogger.Error("插件", triggerLocalPath, "中的Topic方法必须是(func() string):", err)
			continue
		}
		topic2 := topic1()
		rule, err := open.Lookup("Rule")
		if err != nil {
			c_log.GlobalLogger.Error("加载本地插件", triggerLocalPath, "中的Rule方法失败。", err)
			continue
		}

		if TopicOfDiagnostics == topic2 { // 1
			if f, ok1 := rule.(func(*diagnostic_msgs.DiagnosticArray) string); ok1 {
				RuleOfDiagnostics = append(RuleOfDiagnostics, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfImu == topic2 { // 2
			if f, ok1 := rule.(func(data *sensor_msgs.Imu) string); ok1 {
				RuleOfImu = append(RuleOfImu, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfLocateInfo == topic2 { // 3
			if f, ok1 := rule.(func(data *pjibot_patrol_msgs.LocateInfo) string); ok1 {
				RuleOfLocateInfo = append(RuleOfLocateInfo, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfObstacleDetection == topic2 { // 4
			if f, ok1 := rule.(func(data *std_msgs.UInt8) string); ok1 {
				RuleOfObstacleDetection = append(RuleOfObstacleDetection, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfOdom == topic2 { // 5
			if f, ok1 := rule.(func(data *nav_msgs.Odometry) string); ok1 {
				RuleOfOdom = append(RuleOfOdom, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfSysInfo == topic2 { // 6
			if f, ok1 := rule.(func(data *pjibot_patrol_msgs.SysInfo) string); ok1 {
				RuleOfSysInfo = append(RuleOfSysInfo, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfRobotPose == topic2 { // 7
			if f, ok1 := rule.(func(data *geometry_msgs.PoseStamped) string); ok1 {
				RuleOfRobotPose = append(RuleOfRobotPose, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfTaskFeedbackInfo == topic2 { // 8
			if f, ok1 := rule.(func(data *pjibot_patrol_msgs.TaskFeedbackInfo) string); ok1 {
				RuleOfTaskFeedbackInfo = append(RuleOfTaskFeedbackInfo, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else if TopicOfWheelOdom == topic2 { // 9
			if f, ok1 := rule.(func(data *nav_msgs.Odometry) string); ok1 {
				RuleOfWheelOdom = append(RuleOfWheelOdom, f)
				goto JudgeDone
			}
			log(triggerLocalPath)
			continue
		} else {
			c_log.GlobalLogger.Error("未知的topic：", topic2)
			continue
		}

	JudgeDone:
		label, err := open.Lookup("Label")
		if err != nil {
			c_log.GlobalLogger.Error("加载本地插件 ", triggerLocalPath, " 中的 Label 方法失败。", err)
			continue
		}
		labelFunc := label.(func() string)
		labelString := labelFunc()
		LabelMapTriggerId.Store(labelString, triggerId)
		success++
	}
	c_log.GlobalLogger.Info("一共应加载", len(config.PlatformConfig.TaskTriggers), "个触发器，实际加载 ", success, " 个触发器。")
}
func log(triggerLocalPath string) {
	c_log.GlobalLogger.Error("插件", triggerLocalPath, "中的 Rule 方法参数格式不正确。")
}
