package main

import (
	"fmt"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/geometry_msgs"
	"math"
)

func Topic() string {
	return "/robot_pose"
}

// 禁止存在下划线_
func Label() string {
	return "PassManholeCover"
}

type Point struct {
	X float64
	Y float64
}

var (

	//定义园区部分井盖ＵＴＭ坐标
	point3 = Point{-29.5885 + 88.96626338170609, 108.087 + 40.553671177476645}
	point4 = Point{209.923 + 88.96626338170609, 210.394 + 40.553671177476645}
	point5 = Point{16.441 + 88.96626338170609, -37.0252 + 40.553671177476645}
	point6 = Point{-53.9421 + 88.96626338170609, -74.9124 + 40.553671177476645}
	point7 = Point{232.502 + 88.96626338170609, 172.084 + 40.553671177476645}

	pointlist = []Point{point3, point4, point5, point6, point7}
)

func Rule(data *geometry_msgs.PoseStamped) string {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("Recovered from panic:", r)
		}
	}()
	//fmt.Println("判断是否覆盖井盖")
	enterFlag := IfEnter(pointlist, 1.5, data.Pose.Position.X, data.Pose.Position.Y)
	if enterFlag {
		return Label()
	}

	return ""
}
func IfEnter(pointList []Point, radius float64, x, y float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{X: x, Y: y}
	for _, point := range pointList {
		d := distance(point1, point)
		if d <= radius {
			return true
		}
	}
	return false
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	d := math.Sqrt((point2.X-point1.X)*(point2.X-point1.X) + (point2.Y-point1.Y)*(point2.Y-point1.Y))

	return d
}
